/*
 * Work under Copyright. Licensed under the EUPL.
 * See the project README.md and LICENSE.txt for more information.
 */

package net.dries007.tfc.util;

import java.util.Arrays;
import javax.annotation.Nullable;

import com.google.common.collect.ImmutableList;
import net.minecraft.block.BlockFalling;
import net.minecraft.block.state.IBlockState;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;

import net.dries007.tfc.objects.entity.EntityFallingBlockTFC;

import net.dries007.tfc.TerraFirmaCraft;
import net.dries007.tfc.objects.entity.EntityFallingBlockTFC;

public interface IFallingBlock
{
    default boolean canFallThrough(IBlockState state)
    {
        return state.func_185904_a().func_76222_j();
    }

    // Can the block fall at a particular position; ignore horizontal falling
    default boolean shouldFall(World world, BlockPos pos)
    {
        return canFallThrough(world.func_180495_p(pos.func_177977_b()));
    }

    // Get the position that the block will fall from (allows for horizontal falling)
    @Nullable
    BlockPos getFallablePos(World world, BlockPos pos);

    default void checkFalling(World worldIn, BlockPos pos, IBlockState state)
    {
        BlockPos pos1 = getFallablePos(worldIn, pos);
        if (pos1 != null)
        {
            if (!BlockFalling.field_149832_M && worldIn.func_175707_a(pos.func_177982_a(-32, -32, -32), pos.func_177982_a(32, 32, 32)))
            {
                if (!pos1.equals(pos))
                {
                    worldIn.func_175698_g(pos);
                    worldIn.func_175656_a(pos1, state);
                }
                worldIn.func_72838_d(new EntityFallingBlockTFC(worldIn, pos1, this, worldIn.func_180495_p(pos1)));
            }
            else
            {
                worldIn.func_175698_g(pos);
                pos1 = pos1.func_177977_b();
                while (canFallThrough(worldIn.func_180495_p(pos1)) && pos1.func_177956_o() > 0)
                    pos1 = pos1.func_177977_b();
                if (pos1.func_177956_o() > 0) worldIn.func_175656_a(pos1.func_177984_a(), state); // Includes Forge's fix for data loss.
            }
        }
    }

    default Iterable<ItemStack> getDropsFromFall(World world, BlockPos pos, IBlockState state, @Nullable NBTTagCompound teData, int fallTime, float fallDistance)
    {
        return ImmutableList.of(new ItemStack(state.func_177230_c(), 1, state.func_177230_c().func_180651_a(state)));
    }

    default void onEndFalling(World world, BlockPos pos, IBlockState state, IBlockState current) {}
}
